#include "container.h"
#include "macros.h"
#include "fs_d64_funcs.h"
#include "fs_sys_funcs.h"
#include "debug_funcs.h"
#include "dirent_funcs.h"
#include "path_funcs.h"
#include "string_funcs.h"
#include "file_funcs.h"

static const char* container_name[] = {
    "sys",
    "d64",
};

const char* container_get_name(uint8_t type) {
    if(type >= ARRAY_ELEMS(container_name)) return "unknown";
    return container_name[type];
}

int container_get_type(char* path, uint8_t* type) {
    /* XXX TODO also stat file, to check if it exists, and check that it is not a dir */
    if(strlen(path) > 4 && strcasecmp(path + strlen(path) - 4, ".d64") == 0) {
        *type = CTYPE_D64;
        return ERR_OK;
    } else {
        *type = CTYPE_SYS;
        return ERR_OK;
    }
}

int container_open(file64* f, char* path) {
    int err = ERR_OK;
    container* cnt = f->cnt;
    debug_message("opening corresponding container '%s'\n", f->path);
    container_get_type(f->path, &f->media);
    if(f->media == CTYPE_SYS) {
        debug_message("container is ufs, no need to open\n");
        cnt->open = 1;
    }
    else {
        if(!cnt->open) {
            if((cnt->file = fopen(f->path,"rb+")) == NULL) {
                err = ERR_DRIVE_NOT_READY;
                debug_message("container open failed: '%s'\n", strerror(errno));
            }
            else {
                cnt->open = 1;
            }
        }
        else {
            debug_message("diskimage already opened\n");
        }
    }
    if(cnt->open) {
        //debug_message("filedescriptor is '%d'\n",cnt->file->_fileno);
        /* get header info */
        switch(f->media) {
            case CTYPE_SYS:
                if((err = fs_sys_get_header(cnt, path))) return err;
                if((err = fs_sys_get_free(cnt))) return err;
            break;
            case CTYPE_D64:
                if((err = fs_d64_get_header(cnt, path))) return err;
                if((err = fs_d64_get_free(cnt))) return err;
            break;
            default:
                return ERR_UNKNOWN_CTYPE;
            break;
        }
    }
    return err;
}

int container_close(file64* f) {
    debug_message("closing corresponding container\n");
    if(f->cnt->open) {
        if(f->cnt->file) fclose(f->cnt->file);
        f->cnt->file = NULL;
        f->cnt->open = 0;
    }
    return ERR_OK;
}

void container_destruct(container* cnt) {
    int i;
    if(!cnt) return;
    if(cnt->dirlist != NULL) {
        debug_message("freeing %d direntries...\n", cnt->dirsize);
        for(i = 0; i < cnt->dirsize; i++) {
            dirent_destruct((cnt->dirlist)[i]);
            (cnt->dirlist)[i] = NULL;
        }
        free(cnt->dirlist);
        cnt->dirlist = NULL;
    }
    cnt->dirsize = 0;
    cnt->dirpos  = 0;

    if(cnt->header) free(cnt->header);

    free(cnt->id);
    free(cnt->bam);
    free(cnt->sectbuf);
    free(cnt);
}

container* container_create() {
    container* cnt = calloc(1, sizeof(container));
    cnt->dirlist = NULL;
    cnt->dir = NULL;
    cnt->file = NULL;
    cnt->header = NULL;
    cnt->id = calloc(6, sizeof(char));
    cnt->bam = calloc(256, sizeof(char));
    cnt->sectbuf = calloc(256, sizeof(char));
    return cnt;
}

void container_init(container* cnt) {
    cnt->dirsize = 0;
    cnt->dirpos  = 0;
    cnt->sectpos  = 0;
    cnt->sectsize = 0;
    cnt->err = ERR_OK;
    cnt->err_track = 0;
    cnt->err_sector = 0;
    cnt->open = 0;
}
